/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is 
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Joe Hewitt <hewitt@netscape.com>
 *                 HJ van Rantwijk <bugs4hj@netscape.net>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or 
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gConsole, gConsoleBundle, gPrefs;

/* :::::::: Console Initialization ::::::::::::::: */

window.onload = function()
{
  gConsole = document.getElementById("ConsoleBox");
  gConsoleBundle = document.getElementById("ConsoleBundle");
  
  top.controllers.insertControllerAt(0, ConsoleController);
  
  updateSortCommand(gConsole.sortOrder);
  updateModeCommand(gConsole.mode);
  updateFilterCommand(gConsole.filter);

  if (!gPrefs) {
    gPrefs = Components.classes['@mozilla.org/preferences-service;1']
                       .getService(Components.interfaces.nsIPrefBranch);
  }
  var iframe = document.getElementById("Evaluator");
  iframe.addEventListener("load", displayResult, true);
}

/* :::::::: Console UI Functions ::::::::::::::: */

function changeMode(aMode)
{
  switch (aMode) {
    case "Errors":
    case "Warnings":
    case "Messages":
      gConsole.mode = aMode;
      break;
    case "All":
      gConsole.mode = null;
  }
  
  document.persist("ConsoleBox", "mode");
}

function changeFilter(aFilter)
{
  switch (aFilter) {
    case "CHROME":
    case "HTTP":
    case "HTTPS":
    case "FILE":
      gConsole.filter = aFilter.toLowerCase();
      break;
    case "OFF":
      gConsole.filter = null;
  }
  document.persist("ConsoleBox", "filter");
}

function checkChromePref()
{
  var showChromeErrors = true;
  var broadcastCHROME = document.getElementById("Console:allowCHROME");

  try {
    showChromeErrors = gPrefs.getBoolPref("javascript.options.showInConsole");
  } 
  catch(ex) {}

  if (!showChromeErrors) {
    broadcastCHROME.setAttribute("disabled", "true");
    if (gConsole.filter == "chrome")
      updateFilterCommand("OFF");
  }
  else broadcastCHROME.removeAttribute("disabled");
  gConsole._showChromeErrors = (showChromeErrors == true);
}

function clearConsole()
{
  gConsole.clear();
}

function changeSortOrder(aOrder)
{
  updateSortCommand(gConsole.sortOrder = aOrder);
}

function updateSortCommand(aOrder)
{
  var orderString = aOrder == 'reverse' ? "Descend" : "Ascend";
  var bc = document.getElementById("Console:sort"+orderString);
  bc.setAttribute("checked", true);  

  orderString = aOrder == 'reverse' ? "Ascend" : "Descend";
  bc = document.getElementById("Console:sort"+orderString);
  bc.setAttribute("checked", false);
}

function updateModeCommand(aMode)
{
  var bc = document.getElementById("Console:mode" + aMode);
  bc.setAttribute("checked", true);
}

function updateFilterCommand(aFilter)
{
  aFilter = aFilter.toUpperCase();
  var bc = document.getElementById("Console:allow" + aFilter);
  bc.setAttribute("checked", true);
}

function toggleToolbar(aEl)
{
  var bc = document.getElementById(aEl.getAttribute("observes"));
  var truth = bc.getAttribute("checked");
  bc.setAttribute("checked", truth != "true");
  var toolbar = document.getElementById(bc.getAttribute("_toolbar"));
  toolbar.setAttribute("hidden", truth);

  document.persist(toolbar.id, "hidden");
  document.persist(bc.id, "checked");
}

function copyItemToClipboard()
{
  gConsole.copySelectedItem();
}

function viewSourceIn(aTarget)
{
  if (gConsole.selectedItem) { // we need a selected item or it fails!
    var url = gConsole.selectedItem.getAttribute("url");
    var line = gConsole.selectedItem.getAttribute("line");
    var noNavigatorWindow = (getTabbrowser(false) == null) ? true : false;
    var tabbrowser = getTabbrowser(true);

    if (url) { // skip for messages, no url there
      switch (aTarget) {
        case "new_tab":
             var browser = null;

             if (tabbrowser) {
               // Prevent url/tab duplication
               var tab = preventDupeTab('view-source:'+url, false, tabbrowser);
               // Is this url already loaded?
               if (tab) { // Yes, activate that tab and bail out!
                 tabbrowser.selectedTab = tab;
                 var win = getNavigatorWindow(false);
                 win.focus();
                 return;
               }
               // Prevent skipping of blank tabs
               browser = checkForBlankTab(tabbrowser);
               if (browser)
                 tab = tabbrowser.getTabForBrowser(browser);
               else
                 tab = tabbrowser.addTab(null, null, null, "restoreTab", null);
               browser = tabbrowser.getBrowserForTab(tab);
               tabbrowser.viewSourceInTab(browser, url, line)
             }
           break;
        case "new_window":
             if (noNavigatorWindow) // JS Console only open
               tabbrowser = getTabbrowserFromWindow(true); // open a new (first) navigator window
             else { // Did we have an open navigator window?
               window.open(); // Yes, open a new (second/third/...) navigator window
               tabbrowser = getTabbrowser(false);
             }
             tabbrowser.viewSourceInTab(tabbrowser.getBrowserAtIndex(0), url, line);
             break;
        default: // View Source file in default 'view source' window
             window.openDialog("chrome://navigator/content/viewSource.xul", "", "scrollbars,resizable,chrome,dialog=no", url, null, null, line);
      }
    }
  }
}

function isItemSelected()
{
  return gConsole.selectedItem != null;
}

function isSourceLink()
{
  return gConsole.sourceLink != null;
}

function UpdateCopyMenu()
{
  goUpdateCommand("cmd_copy");
}

function UpdateMenuItems()
{
  goUpdateCommand("cmd_copy");
  goUpdateCommand("cmd_viewSource");
  goUpdateCommand("cmd_viewSourceInTab");
  goUpdateCommand("cmd_viewSourceInWindow");
}

function onEvalKeyPress(aEvent)
{
  if (aEvent.keyCode == 13)
    evaluateTypein();
}

function evaluateTypein()
{
  var code = document.getElementById("TextboxEval").value;
  var iframe = document.getElementById("Evaluator");
  iframe.setAttribute("src", "javascript: " + code);
}

function displayResult()
{
  var resultRange = Evaluator.document.createRange();
  resultRange.selectNode(Evaluator.document.documentElement);
  var result = resultRange.toString();
  if (result)
    gConsole.mCService.logStringMessage(result);
    // or could use appendMessage which doesn't persist
  var iframe = document.getElementById("Evaluator");
  if (iframe.getAttribute("src") != "about:blank")
    iframe.setAttribute("src", "about:blank");
}

/* :::::::: Command Controller for the Window ::::::::::::::: */

var ConsoleController = 
{
  isCommandEnabled: function (aCommand)
  {
    switch (aCommand) {
      case "cmd_copy":
        return isItemSelected();
      case "cmd_viewSource":
      case "cmd_viewSourceInTab":
      case "cmd_viewSourceInWindow":
        return isSourceLink();
      default:
        return false;
    }
  },
  
  supportsCommand: function (aCommand) 
  {
    switch (aCommand) {
      case "cmd_copy":
      case "cmd_viewSource":
      case "cmd_viewSourceInTab":
      case "cmd_viewSourceInWindow":
        return true;
      default:
        return false;
    }
  },
  
  doCommand: function (aCommand)
  {
    switch (aCommand) {
      case "cmd_copy":
        copyItemToClipboard();
        break;
      case "cmd_viewSource":
        viewSourceIn("default");
        break;
      case "cmd_viewSourceInTab":
        viewSourceIn("new_tab");
        break;
      case "cmd_viewSourceInWindow":
        viewSourceIn("new_window");
        break;
      default:
        break;
    }
  },
  
  onEvent: function (aEvent) 
  {
  }
};

// XXX DEBUG

function debug(aText)
{
  var csClass = Components.classes['@mozilla.org/consoleservice;1'];
  var cs = csClass.getService(Components.interfaces.nsIConsoleService);
  cs.logStringMessage(aText);
}

function getStackTrace()
{
  var frame = Components.stack.caller;
  var str = "";
  while (frame) {
    if (frame.filename)
      str += frame.filename + ", Line " + frame.lineNumber;
    else
      str += "[" + gConsoleBundle.getString("noFile") + "]";
    
    str += " --> ";
    
    if (frame.functionName)
      str += frame.functionName;
    else
      str += "[" + gConsoleBundle.getString("noFunction") + "]";
      
    str += "\n";
    
    frame = frame.caller;
  }
  
  return str;
}

////////////////////////////////////////////////////////////////////////////////
// Start of 'helper functions' copied over from the MultiZilla source code!
////////////////////////////////////////////////////////////////////////////////
function preventDupeTab(href, useFirstPage, aBrowser)
{
  if (gPrefs.getBoolPref("multizilla.prevent.double") && href.substring(0,10) != "javascript") {
    if (!aBrowser)
      aBrowser = getBrowser();
    var tabHref = null;
    var baseHref = href.replace(/#.*/, "");
    if (baseHref[baseHref.length-1] == '/')
      baseHref = baseHref.substring(0, baseHref.length-1);
    for (var i = 0; i < aBrowser.mTabContainer.childNodes.length; i++) {
      if (aBrowser.mTabContainer.childNodes[i].hasAttribute("busy") || useFirstPage)
        tabHref = aBrowser.mTabContainer.childNodes[i].getAttribute("firstPage");
      else { 
        tabHref = aBrowser.getBrowserAtIndex(i).webNavigation.currentURI.spec;
      }
      tabHref = tabHref.replace(/#.*/, "");
      if (tabHref[tabHref.length-1] == '/')
        tabHref = tabHref.substring(0, tabHref.length-1);
      var tab = aBrowser.mTabContainer.childNodes[i];
      if (baseHref == tabHref) {
        if (aBrowser.selectedTab == tab)
          aBrowser.loadURI(href); // New 'aBrowser' added!
        else {
          aBrowser.selectedTab = tab;
          if (href.indexOf("#") != -1)
            aBrowser.loadURI(href); // anchor link like webpage.html#anchor
        }
        return tab;
      }
    }
  }
  return null;
}
////////////////////////////////////////////////////////////////////////////////
function checkForBlankTab(aBrowser)
{
  if (gPrefs.getBoolPref("multizilla.tabs.blank-tab-first")) {
    var page = "about:blank";
    var browser = aBrowser || getBrowser();
    var numberOfTabs = browser.mTabContainer.childNodes.length;

    for (var i = 0; i < numberOfTabs; i++) {
      if (browser.mTabContainer.childNodes[i].hasAttribute("busy"))
        page = browser.mTabContainer.childNodes[i].getAttribute("firstpage");
      else 
        page = browser.getBrowserAtIndex(i).currentURI.spec

      if (page == "about:blank")
        return browser.getBrowserAtIndex(i)
    }
  }
  return null;
}
////////////////////////////////////////////////////////////////////////////////
function getNavigatorWindow(aOpenFlag)
{
  var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
  var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
  var navigatorWindow = windowManagerInterface.getMostRecentWindow("navigator:browser");

  if (!navigatorWindow) {
    if (!aOpenFlag)
      return null;
    else {
      window.open();
      navigatorWindow = windowManagerInterface.getMostRecentWindow("navigator:browser");
    }
  }
  return navigatorWindow;
}
////////////////////////////////////////////////////////////////////////////////
function getTabbrowser(aOpenFlag)
{
  var navigatorWindow = getNavigatorWindow(aOpenFlag);

  if (navigatorWindow)
    return navigatorWindow.getBrowser();
  return null;
}

