/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - March 11, 2003.
   -
   - The Initial Developer of the Original Code is HJ van Rantwijk.
   - Portions created by HJ van Rantwijk are Copyright (C) 2003-2005
   - HJ van Rantwijk.  All Rights Reserved.
   -
*/

////////////////////////////////////////////////////////////////////////////////
function BrowserFullScreen()
{
  var MENUBAR = 0;

  var NAVTOOLBAR = 0;
  var PERSONALTOOLBAR = 1;
  var LINKTOOLBAR = 2;

  var TABMODETOOLBAR = 0;
  var SKIPTOOLBARS = 2;
  var STATUSBAR = 0;

  var fullScreenEnabled = mzPreferences.readMyPref("multizilla.fullscreen.enabled", "bool", true);
  var chromeHidden = document.documentElement.hasAttribute("chromehidden");

  if (chromeHidden) {
    if (mzPreferences.readMyPref("multizilla.fullscreen.re-enable-chrome", "bool", true)) {
      document.documentElement.removeAttribute("chromehidden");
      return;
    }
    if (mzPreferences.readMyPref("multizilla.fullscreen.disable.chromeless", "bool", true))
      return;
  }
  if (fullScreenEnabled) {
    var firstBrowserOnly = mzPreferences.readMyPref("multizilla.fullscreen.first-browserwindow-only", "bool", true);

    if (firstBrowserOnly && !mzIsFirstWindow())
      return;
    if (!window.fullScreen) {
      var XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
      // Menubar
      var prefValue = mzPreferences.readMyPref("multizilla.fullscreen.enable.menubar", "bool", false);
      var menubars = document.getElementsByTagNameNS(XULNS, "menubar");
      var menubar = menubars[MENUBAR];
      menubar.setAttribute("fullscreentoolbar", prefValue ? "true" : "false");

      var toolbars = document.getElementsByTagNameNS(XULNS, "toolbar");

      // Navigation Toolbar
      prefValue = mzPreferences.readMyPref("multizilla.fullscreen.enable.nav-bar", "bool", false);
      var toolbar = toolbars[NAVTOOLBAR];
      toolbar.setAttribute("fullscreentoolbar", prefValue ? "true" : "false");

      // Personal Toolbar
      prefValue = mzPreferences.readMyPref("multizilla.fullscreen.enable.personaltoolbar", "bool", false);
      toolbar = toolbars[PERSONALTOOLBAR];
      toolbar.setAttribute("fullscreentoolbar", prefValue ? "true" : "false");

      // Site Navigation Bar
      prefValue = mzPreferences.readMyPref("multizilla.fullscreen.enable.linktoolbar", "bool", false);
      toolbar = toolbars[LINKTOOLBAR];
      toolbar.setAttribute("fullscreentoolbar", prefValue ? "true" : "false");

      // Third Party Toolbars
      if (toolbars.length > SKIPTOOLBARS) {
        for (var i = SKIPTOOLBARS; i < toolbars.length-1; i++) {
          var id = "Toolbar-" + (i - SKIPTOOLBARS + 2);

          if (toolbars[i].hasAttribute("id"))
            id = toolbars[i].getAttribute("id");

          try {
            prefValue = mzPreferences.readMyPref("multizilla.fullscreen.enable.thirdparty."+id, "bool", false);
            toolbars[i].setAttribute("fullscreentoolbar", prefValue ? "true" : "false");
          } 
          catch(ex) {
            // Disable Third Party Toolbars by default
            toolbars[i].setAttribute("fullscreentoolbar", "false");

            if (id)
              mzPreferences.writePref("bool", "multizilla.fullscreen.enable.thirdparty."+id, false);
		  }
        }          
      }
      // MultiZilla Toolbar
      prefValue = mzPreferences.readMyPref("multizilla.fullscreen.enable.tabmodetoolbar", "bool", true);
      toolbars = document.getElementsByTagNameNS(XULNS, "tabmodetoolbar");
      toolbar = toolbars[TABMODETOOLBAR];
      toolbar.setAttribute("fullscreentoolbar", prefValue ? "true" : "false");

      // Status Bar
      prefValue = mzPreferences.readMyPref("multizilla.fullscreen.enable.status-bar", "bool", true);
      toolbars = document.getElementsByTagNameNS(XULNS, "statusbar");
      toolbar = toolbars[STATUSBAR];
      toolbar.setAttribute("fullscreentoolbar", prefValue ? "true" : "false");
    }
    // Toggle FullScreen
    window.fullScreen = !window.fullScreen;
  }
  else { // Ask users permission to re-enable FullScreen mode
    var aHeader = mzGetTextFromBundle("reenableFSHeader");
    var aQuestion = mzGetTextFromBundle("reenableFSQuestion");
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);

    if (promptService.confirm(window, aHeader, aQuestion)) {    
       mzPreferences.writePref("bool", "multizilla.fullscreen.enabled", true);
       BrowserFullScreen();
    }
  }
}
////////////////////////////////////////////////////////////////////////////////
FullScreen.toggle = function() 
{
  this.showXULChrome("menubar", window.fullScreen);
  this.showXULChrome("toolbar", window.fullScreen);
  this.showXULChrome("tabmodetoolbar", window.fullScreen);
  this.showXULChrome("statusbar", window.fullScreen);
}
////////////////////////////////////////////////////////////////////////////////
FullScreen.showXULChrome = function(aTag, aShow)
{
  var XULNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
  var els = document.getElementsByTagNameNS(XULNS, aTag);

  for (var i = 0; i < els.length; ++i) {
    if (els[i].getAttribute("fullscreentoolbar") == "true") {
      // XXX don't interfere with previously collapsed toolbars
      if (aTag == "toolbar" && els[i].hasAttribute("id")) {
        var id = els[i].getAttribute("id");

        if (id != "nav-bar" && "PersonalToolbar" && "linktoolbar") {
          var collapsed = els[i].getAttribute("moz-collapsed");
          // alert(id+" : "+collapsed);
          continue;
        }
      }
      this.setToolbarButtonMode(els[i], aShow ? "" : "small");
    } else {
      // use moz-collapsed so it doesn't persist hidden/collapsed,
      // so that new windows don't have missing toolbars
      if (aShow) {
        els[i].removeAttribute("moz-collapsed");
      }
      else {
        els[i].setAttribute("moz-collapsed", "true");
      }
    }
  }
  var controls = document.getElementsByAttribute("fullscreencontrol", "true");

  for (i = 0; i < controls.length; ++i)
    controls[i].hidden = aShow;
}
