/* 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.1 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   -
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   -
   - The Original Code is this file as it was released on
   - November 20, 2003.
   -
   - The Initial Developer of the Original Code is HJ van Rantwijk.
   - Portions created by HJ van Rantwijk are Copyright (C) 2003-2005
   - HJ van Rantwijk.  All Rights Reserved.
   -
*/

const MZ_TRUE = "true";
const MZ_FALSE = "false";

const TITLE_PROPERTY    = 0;
const LOCATION_PROPERTY = 4;
const SELECTED_PROPERTY = 6;

var gDialogManager, gHeight, gTabbrowser, gModified;
var gBackLabel, gBackImage, gForwardLabel, gForwardImage;
var gSelectedTabMenulist, gHistoryDataSource, gHistoryTree, gSelectedTabResource = "";
var gHistoryPropertiesToBeRemoved = new Array();
var reloadInterval, helpPages, dataCollection = null;
var elementArray = new Array();
var elementIDs = new Array("label", "custom", "firstpage", "image", "location", "interval", "selected",
                           "ordinal", "allowPlugins", "allowJavascript", "allowMetaRedirects", 
                           "allowSubframes", "allowImages", 
                           "interval-enabled", "allow-all", "suppress-all");

var defaultValues = ["about:blank", "false", "about:blank", "", "about:blank", "0", "false", "0", "true", "true", "true", "true", "true"];

function Startup(aDialog)
{
  gDialogManager = window.arguments[0];
  gDialogManager.selectWindowStyling(aDialog);
  gHeight = (gDialogManager.innerHeight / 50 - 2);

  document.getElementById('mac_button_spring').hidden = !(/Mac/.test(navigator.platform));

  if (gDialogManager.gUseStackedWindows) {
    document.documentElement.setAttribute("hidechrome", "true");
    window.outerWidth = gDialogManager.innerWidth;
    window.outerHeight = 0;
    slideOpen(0);
  }
  else {
    window.outerWidth = gDialogManager.outerWidth;
    window.outerHeight = gDialogManager.outerHeight;
    document.getElementById("main-box").collapsed = false;
  }
  gTabbrowser = gDialogManager.getTabbrowser(true);
  gBackImage = document.getElementById('BackImage');
  gBackLabel = document.getElementById('BackLabel');
  gForwardLabel = document.getElementById('ForwardLabel');
  gForwardImage = document.getElementById('ForwardImage');
  gSelectedTabMenulist = document.getElementById("SelectedTabMenulist");
  initTabProperties();
  selectDefaultButton();
  enableBookmarkButton();
  setResource(0);
  gHistoryDataSource = Components.classes['@mozilla.org/rdf/datasource;1?name=in-memory-datasource'].createInstance(Components.interfaces.nsIRDFDataSource);
  gHistoryTree = document.getElementById("TabHistoryTree");
  gHistoryTree.database.AddDataSource(gHistoryDataSource);
  initHistoryTree(gDialogManager.gResource);
  initTabSelectorMenu();

  var selectedIndex = gSelectedTabMenulist.selectedIndex;

  if (selectedIndex < 0) {
    gSelectedTabMenulist.selectedIndex = 0;
    selectedIndex = 0;
    var resource = gSelectedTabMenulist.firstChild.childNodes[selectedIndex].getAttribute("resource");
    changeSelectedTabTo(resource);
    hasModifiedSettings();
  }
  else 
    gSelectedTabResource = gSelectedTabMenulist.firstChild.childNodes[selectedIndex].getAttribute("resource");
  // Init tab popup menu for ctrl+clicks on image buttons
  addTabsToMenu(document.getElementById("TabsPopup"));
  refreshFocus();
}

function slideOpen(aStep)
{
  if (aStep < gHeight) {
    window.outerHeight += 50;
    setTimeout(slideOpen, 20, ++aStep);
  } 
  else {
    /* var defaultHeight = gDialogManager.document.documentElement.getAttribute('inner-height');

    if (gDialogManager.innerHeight < defaultHeight)
      gDialogManager.innerHeight = defaultHeight; */
    window.outerHeight = gDialogManager.innerHeight;
    document.getElementById("main-box").collapsed = false;
  }
}

function slideClosed(aStep)
{
  gHistoryTree.database.RemoveDataSource(gHistoryDataSource);
  document.getElementById("dialogButtonBox").collapsed = true;

  if (gDialogManager.gUseStackedWindows && aStep < gHeight) {
    window.outerHeight -= 50;
    setTimeout(slideClosed, 10, ++aStep);
  } 
  else
    window.close();
}

function initTabProperties()
{
  // Is this the first run?
  if (typeof elementIDs[0] == 'string') {
    // Yes, initialize element array (used in activateSettings)
    for (i in elementIDs) {
      if (i == 6) // skip 'selected'
        continue;
      elementArray[i] = document.getElementById(elementIDs[i]);
    }
  }
  if (mzRDFStore == null)
    initializeRDFStorageFile();
  // Initialize textboxes, checkboxes and buttons
  dataCollection = getTabDataFromResource(mzRDFStore, RDF.GetResource(gDialogManager.gResource));

  if (dataCollection) {
    activateSettings(dataCollection);
    // Initialize the 'Allow All' or 'Suppress All' buttons
    onProperty(null);
  }
  setBackForwardButtons();
}

function activateSettings(aDataCollection)
{
  elementArray[0].value = aDataCollection[0]; // title
  elementArray[1].checked = (aDataCollection[1] == MZ_TRUE); // custom
  elementArray[2].value = aDataCollection[2]; // firstpage
  elementArray[3].src = aDataCollection[3]; // favicon/site icon
  elementArray[3].hidden = (aDataCollection[3] == "");
  elementArray[4].value = aDataCollection[4]; // location
  reloadInterval = aDataCollection[5]; // interval
  elementArray[5].disabled = (reloadInterval == 0);
  gTabbrowser.initReloadPopup(elementArray[5].firstChild, null, reloadInterval);
  // Tab Properties
  for (var i = 8; i < 13; i++) {
    elementArray[i].checked = (aDataCollection[i] == MZ_TRUE);
  }  
  elementArray[13].checked = (reloadInterval > 0);
}

function getDataCollection()
{
  var defaultScheme = "http://";
  var aDataCollection = new Array();
  // Data Collection buildup
  aDataCollection[0] = elementArray[0].value; // title
  aDataCollection[1] = elementArray[1].checked ? MZ_TRUE : MZ_FALSE; // custom
  // Check for missing scheme in 'firstpage' textbox.
  if (elementArray[2].value.indexOf(":") < 0) // Add default scheme
    elementArray[2].value = defaultScheme + elementArray[2].value;
  if (elementArray[2].value == defaultScheme)
    elementArray[2].value = defaultValues[2];
  aDataCollection[2] = elementArray[2].value; // firstpage
  aDataCollection[3] = elementArray[3].value; // image
  // Check for missing scheme in 'location' textbox.
  if (elementArray[4].value.indexOf(":") < 0) // Add default scheme
    elementArray[4].value = defaultScheme + elementArray[4].value;
  if (elementArray[4].value == defaultScheme)
    elementArray[4].value = defaultValues[4];
  aDataCollection[4] = elementArray[4].value; // location
  aDataCollection[5] = reloadInterval; // Reload interval

  // Tab Properties
  for (var i = 8; i < 13; i++) {
    aDataCollection[i] = elementArray[i].checked ? MZ_TRUE : MZ_FALSE;
  }
  return aDataCollection;
}

function refreshReloadState()
{
  var disabled = !elementArray[13].checked
  elementArray[5].disabled = disabled;

  if (disabled) {
    elementArray[5].value = 0;
    reloadInterval = 0;
    hasModifiedSettings();
  }
}

function onProperty(aElement)
{
  if (aElement) // skipped at startup
    hasModifiedSettings();
  var id = aElement ? aElement.getAttribute("id") : "";
  var properties = 0;
  // Count number of checked property checkboxes
  for (var i = 8; i < 13; i++) {
    if (elementArray[i].checked)
      properties++
  }
  // Called from the 'Allow All' or 'Suppress All' buttons
  if (id == elementIDs[14] || id == elementIDs[15]) {
    properties = (id == elementIDs[14]) ? 5 : 0;
    for (i = 8; i < 13; i++)
      elementArray[i].checked = (properties);
  }
  // Update button state
  elementArray[14].disabled = (properties == 5);
  elementArray[15].disabled = (properties == 0);
}

function setReloadInterval(aMenuitem)
{
  hasModifiedSettings();
  reloadInterval = (aMenuitem.getAttribute('value') * 1000);
}

function onCancel()
{
  slideClosed(0);
}

function onAccept()
{
  storeDataCollection();
  slideClosed(0);
}

function storeDataCollection()
{
  var MZ = "http://multizilla.mozdev.org/rdf#";
  var modifiedDataCollection = getDataCollection();
  var resource = RDF.GetResource(gDialogManager.gResource);

  try {
    for (var i = 0; i < 13; i++) {
      if (modifiedDataCollection[i] != undefined && dataCollection[i] != modifiedDataCollection[i]) {
        var property = RDF.GetResource(MZ+elementIDs[i]);
        var newValue = RDF.GetLiteral(modifiedDataCollection[i]);
        var currentValue = mzRDFStore.GetTarget(resource, property, true);

        if (modifiedDataCollection[i] == defaultValues[i])
          mzRDFStore.Unassert(resource, property, currentValue);
        else
          addOrChangeValue(mzRDFStore, resource, property, currentValue, newValue);
      }
    }
  } catch(ex) { 
    // die silently
  }
  // Are there pending history items for removal?
  if (gHistoryPropertiesToBeRemoved.length > 0) {
    for (i in gHistoryPropertiesToBeRemoved) {
      mzRemoveHistoryByObjects(gHistoryPropertiesToBeRemoved[i]);
    }
  }
  resource = gSelectedTabMenulist.firstChild.childNodes[gSelectedTabMenulist.selectedIndex].getAttribute("resource");

  if (resource && resource != gSelectedTabResource)
    changeSelectedTabTo(resource);
  else 
    mzFlushDataSource(mzRDFStore);

  gModified = false;
  modifiedDataCollection = null;
  gHistoryPropertiesToBeRemoved = new Array();
}

function hasModifiedSettings(aState)
{
  var acceptButton = document.documentElement.getButton('accept');
  gModified = (aState == "no") ? false : true;
  acceptButton.disabled = !gModified;
  var cancelButton = document.documentElement.getButton('cancel');
  cancelButton.label = gDialogManager.stringBundle.getString('prefPanelCancelButton.label');
  selectDefaultButton();
}

function selectDefaultButton()
{
  var acceptButton = document.documentElement.getButton('accept');
  var cancelButton = document.documentElement.getButton('cancel');
  var defaultButton = cancelButton;

  acceptButton.removeAttribute('default');
  cancelButton.removeAttribute('default');

  if (gModified)
    defaultButton = acceptButton;
  else
    acceptButton.disabled = true;
  defaultButton.setAttribute('default', 'true');
}

function setBackForwardButtons()
{
  var tree = gDialogManager.tree;
  var row = tree.currentIndex;
  var disabled = (row > 0) ? tree.treeBoxObject.view.isContainer(row - 1) : false;

  gBackLabel.disabled = disabled;
  gBackImage.setAttribute('disabled', disabled ? 'true' : 'false');
  disabled = (tree.treeBoxObject.view.hasNextSibling(row, row) == false);
  gForwardLabel.disabled = disabled;
  gForwardImage.setAttribute('disabled', disabled ? 'true' : 'false');
}

function setResource(aDirection, aRow)
{
  var tree = gDialogManager.tree;
  tree.currentIndex = (aRow == undefined) ? (tree.currentIndex + aDirection) : aRow;
  gDialogManager.sessionManager.selectRows(tree, 'none');
  tree.treeBoxObject.ensureRowIsVisible(tree.currentIndex);

  var resource = tree.builderView.getResourceAtIndex(tree.currentIndex).Value;
  document.getElementById('TabSessionID').value = resource;
  gDialogManager.gResource = resource;
}

function refreshPropertyData(aDirection, aRow)
{
  // We need to check the disable state of the 'label' buttons!
  var canBack = gBackLabel.disabled;
  var canForward = gForwardLabel.disabled;

  if (aDirection == -1 && canBack ||
     aDirection == 1 && canForward || 
     aDirection == undefined && aRow == undefined)
    return;

  var prefString = "multizilla.tabs.session-property-changes.confirmation";
  var shouldSkip = gDialogManager.readMyPref(prefString, "bool", false);
  var result = (gModified && shouldSkip) ? 0 : 1;

  if (gModified && !shouldSkip) {
    var dialogTitle = gDialogManager.stringBundle.getString("tsmConfirmActionTitle");
    var dialogText = gDialogManager.stringBundle.getString("tsmConfirmStorageText");
    var checkboxText = gDialogManager.stringBundle.getString("tsmKeepStorageDecisionText");
    var checkbox = {checked:true}
    var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                  .getService(Components.interfaces.nsIPromptService);
    result = promptService.confirmEx(window, dialogTitle, dialogText,
                                     (promptService.BUTTON_POS_0 * promptService.BUTTON_TITLE_YES) + 
                                     (promptService.BUTTON_POS_1 * promptService.BUTTON_TITLE_NO),
                                     null, null, null, checkboxText, checkbox);

    gDialogManager.writePref("bool", prefString, checkbox.value);
  }
  if (result == 0) // Yes returns 0 and No, Esc and X on window returns 1
    storeDataCollection();

  setResource(aDirection, aRow);
  initHistoryTree(gDialogManager.gResource);
  gModified = false;
  selectDefaultButton();
  initTabProperties();
  // Init tab popup menu for ctrl+clicks on image buttons
  addTabsToMenu(document.getElementById("TabsPopup"));
  enableBookmarkButton();
}

function initTabSelectorMenu()
{
  var menulist = document.getElementById("SelectedTabMenulist");
  addTabsToMenu(menulist);
}

function showTabSelectorMenu(aEvent)
{
  var xpos = (aEvent.type == "keyup") ? aEvent.target.boxObject.x : aEvent.clientX;
  var ypos = (aEvent.type == "keyup") ? aEvent.target.boxObject.y : aEvent.clientY;
  var tabsPopup = document.getElementById("TabsPopup");
  tabsPopup.showPopup(tabsPopup, xpos, ypos, "popup", "at_pointer");
}

function addTabsToMenu(aParent)
{
  var selectedResource = gDialogManager.gResource;
  var containerResource = selectedResource.replace(/tab-\w*/, 'root');
  var resourceList = mzGetResourcesFromContainer(mzRDFStore, RDF.GetResource(containerResource));
  var selectedIndex = -1;
  var isMenulist = (aParent.localName == "menulist");
  var popupNode = isMenulist ? aParent.firstChild : aParent; // there's no <menupopup> for <popup>
  var children = popupNode.childNodes;
  var startIndex = -1;
  var resource;

  // Remove old menu items from the bottom up
  for (var i = children.length - 1; i >= 0; --i) {
    popupNode.removeChild(children[i]);
  }
  // The menu items for the tab selection popup need row numbers
  if (!isMenulist) {
    startIndex = gDialogManager.tree.currentIndex;
    // Adjust 'startIndex' to the first tab in this session
    for (i = 0; i < resourceList.length; i++) {
      resource = resourceList[i];
      // Is this the currently displayed tab?
      if (resource == selectedResource) {
        break;
      }
      startIndex--;
    }
  }
  // Add menu items
  for (rIndex in resourceList) {
    resource = resourceList[rIndex];
    var propertyValues = getTabDataFromResource(mzRDFStore, RDF.GetResource(resource));
    var menuitem = document.createElement("menuitem");
    menuitem.setAttribute("class", "menuitem-iconic");
    menuitem.setAttribute("label", propertyValues[0]);
    menuitem.setAttribute("src", propertyValues[3]);
    menuitem.setAttribute("resource", resource);

    if (isMenulist) {
      // Is this the selected tab?
      if (selectedIndex < 0 && propertyValues[6] == MZ_TRUE) {
        menuitem.selected = true;
        selectedIndex = Number(rIndex);
      }
    }
    else {
      var row = startIndex + Number(rIndex);
      menuitem.setAttribute("row", row);
      // Disable the currently selected/displayed tab
      if (resource == selectedResource)
        menuitem.setAttribute("disabled", "true");
    }
    popupNode.appendChild(menuitem);
  }
  if (isMenulist)
    aParent.selectedIndex = selectedIndex;
}
// We might want to rename this function and add it to multiviewsRDF.js for global use
function changeSelectedTabTo(aResource)
{
  var sessionContainerResource = aResource.replace(/tab-\w*/, 'root')
  var resourceList = mzGetResourcesFromContainer(mzRDFStore, RDF.GetResource(sessionContainerResource));
  var selectedTabStyle = gDialogManager.readMyPref("multizilla.session-manager.selectedtab-styling", "string", "bold");
  var mzRDF = "http://multizilla.mozdev.org/rdf#";
  var selected_property = RDF.GetResource(mzRDF + "selected");
  var selectedTabStyle_property = RDF.GetResource(mzRDF + "selectedTabStyle");

  for (i in resourceList) {
    var resource = RDF.GetResource(resourceList[i]);
    var isSelected = mzRDFStore.GetTarget(resource, selected_property, true);
    var hasStyle = mzRDFStore.GetTarget(resource, selectedTabStyle_property, true);

    if (resourceList[i] == aResource) {
      addOrChangeValue(mzRDFStore, resource, selected_property, isSelected, RDF.GetLiteral("true"));
      addOrChangeValue(mzRDFStore, resource, selectedTabStyle_property, hasStyle, RDF.GetLiteral(selectedTabStyle));
    }
    else {
      if (isSelected)
        mzRDFStore.Unassert(resource, selected_property, isSelected);
      if (hasStyle)
        mzRDFStore.Unassert(resource, selectedTabStyle_property, hasStyle);
    }
  }
  mzFlushDataSource(mzRDFStore);
}

function doBackForward(aEvent, aDirection)
{
  if (aEvent.target.getAttribute("disabled") == "true")
    return;

  if (aEvent.ctrlKey)
    showTabSelectorMenu(aEvent);
  else
    refreshPropertyData(aDirection);

  if (aEvent.type == "keyup") {
    if (aEvent.target.getAttribute("disabled") == "true") {
      if (aEvent.target.id == "BackImage")
        document.getElementById("ForwardImage").focus();
      else 
        document.getElementById("BackImage").focus();
    }
    else 
      aEvent.target.focus();
  }
  else 
    refreshFocus();
}

function refreshFocus()
{
  // Focus 'Title' textbox or 'Custom Title' checkbox
  var focusedElement = Number(elementArray[1].checked);
  focusedElement = elementArray[focusedElement];
  // Select all text in the location field
  if (focusedElement.localName == "textbox") {
    focusedElement.setSelectionRange (0, focusedElement.value.length);
    focusedElement.select();
  }
  else
    focusedElement.focus();
}

function selectTab(aEvent)
{
  var menuitem = aEvent.originalTarget;
  var row = Number(menuitem.getAttribute("row"));

  refreshPropertyData(undefined, row);

  var closeTabSelectionMenu = gDialogManager.readMyPref("multizilla.session-manager.tabselectionmenu-autoclose", "bool", true);
  // Some people might want to keep this menu open ;)
  if (closeTabSelectionMenu)
    document.getElementById("TabsPopup").hidePopup();
}

function selectRows(aAction)
{
  var isDisabled = gHistoryTree.hasAttribute("disabled");

  if (isDisabled)
    return;

  if (aAction == 'none')
    gHistoryTree.currentIndex = -1;
  gDialogManager.sessionManager.selectRows(gHistoryTree, aAction);
}

function enableBookmarkButton()
{
  var bookmarks = Components.classes["@mozilla.org/rdf/datasource;1?name=bookmarks"].getService(Components.interfaces.nsIRDFDataSource);
  var URL_Property = RDF.GetResource("http://home.netscape.com/NC-rdf#URL");
  var URL_Literal = RDF.GetLiteral(dataCollection[4]);
  var isBookmarked = bookmarks.GetSource(URL_Property, URL_Literal, true);
  document.getElementById("FileAsBookmark").disabled = (isBookmarked);
}

function fileAsBookmark()
{
  // Ctrl+Shift+D should not work when the 'File As Bookmark' button is disabled
  if (document.getElementById("FileAsBookmark").disabled)
    return;

  var url = elementArray[4].value;
  var title = elementArray[0].value;
  var characterSet = gDialogManager.opener.document.characterSet;

  openDialog("chrome://communicator/content/bookmarks/addBookmark.xul", "",
             "centerscreen, chrome, dialog=yes,resizable=yes,dependent",
             title, url, null, characterSet, null, null);

  refreshFocus();
}

function initHistoryTree(aResource)
{
  var historyObj = getHistoryFromResource(mzRDFStore, aResource);
  var resource = "urn:history:root";

  if (historyObj) {
    removeDataResources(gHistoryDataSource, resource);
    gHistoryTree.removeAttribute("disabled");
    var mzRDF = "http://multizilla.mozdev.org/rdf#";
    var historyContainer = mzConstructRDFContainer(gHistoryDataSource, "seq", resource, false);

    for (hIndex in historyObj.tabHistoryEntries) {
      resource = "urn:history:entry-" + hIndex;
      removeDataResources(gHistoryDataSource, resource);
      var entry = historyObj.tabHistoryEntries[hIndex];
      var title = entry.title || entry.uri.spec;
      var property = mzRDF + "titleOrURI";
      gHistoryDataSource.Assert(RDF.GetResource(resource), RDF.GetResource(property), RDF.GetLiteral(title), true);
      property = mzRDF + "index";
      gHistoryDataSource.Assert(RDF.GetResource(resource), RDF.GetResource(property), RDF.GetLiteral(entry.index), true);
      historyContainer.AppendElement(RDF.GetResource(resource));
      mzFlushDataSource(gHistoryDataSource);
    }
    gHistoryTree.builder.rebuild();
    var historyIndex = Number(historyObj.tabHistoryIndex);

    if (historyIndex >= 0) {
      gHistoryTree.view.selection.clearSelection();
      gHistoryTree.view.selection.select(historyIndex);
      gHistoryTree.treeBoxObject.ensureRowIsVisible(historyIndex);
    }
    historyContainer = null;
  }
  else {
    gHistoryTree.setAttribute("disabled", "true");
  }
  historyObj = null;
}

function initHistoryContextMenu(aEvent)
{
  if (gHistoryTree.hasAttribute("disabled")) // No context menu for disabled trees!
    return false;

  var rowCount = gHistoryTree.treeBoxObject.view.rowCount;
  var targetRow = gDialogManager.sessionManager.getRowNumber(aEvent, gHistoryTree);
  var selectedRows = gDialogManager.sessionManager.getSelectedRows(gHistoryTree);
  // Context menu and menu items
  var contextMenu = document.getElementById("tsmHistoryContextMenu");
  var selectAll = document.getElementById("SelectAll");
  var undoSelection = document.getElementById("UndoSelection");
  var deleteSelectedItems = document.getElementById("DeleteSelectedItems");
  var clearAllItems = document.getElementById("ClearAllItems");
  // The following block needs to be changed/finished!
  if (rowCount >= 1) {
    if (selectedRows.length > 1) {
      selectAll.setAttribute("disabled", "true");
      undoSelection.removeAttribute("disabled");
      deleteSelectedItems.removeAttribute("disabled");
      clearAllItems.setAttribute("disabled", "true");
    }
    else { // No selection, enable "Select All" and disable "Deselect All"
      selectAll.removeAttribute("disabled");
      undoSelection.setAttribute("disabled", "true");

      if (selectedRows.length == 1)
        deleteSelectedItems.removeAttribute("disabled");
      else
        deleteSelectedItems.setAttribute("disabled", "true");
      clearAllItems.removeAttribute("disabled");
    }
  }
  return true;
}

function removeHistoryItems(aEvent)
{
  var action = aEvent.originalTarget.getAttribute("action");
  var selectedResource = gDialogManager.gResource;

  if (action == "all")
    gDialogManager.sessionManager.selectRows(gHistoryTree, action);

  var selectedRows = gDialogManager.sessionManager.getSelectedRows(gHistoryTree);
  gHistoryTree.view.selection.clearSelection();
  var columnID = "historyindex";
  columnID = ('columns' in gHistoryTree) ? gHistoryTree.columns[columnID] : columnID;
  var mzRDF = "http://multizilla.mozdev.org/rdf#";
  // Delete the selected row(s) from the bottom to the top
  for (var i = (selectedRows.length - 1); i > -1; i--) {
    var row = Number(selectedRows[i]);
    var index = gHistoryTree.view.getCellText(row, columnID);
    // Remove selected item from tree
    var resource = "urn:history:entry-" + index;
    removeDataResources(gHistoryDataSource, resource);
    // Remove item from history container
    var historyContainer = mzConstructRDFContainer(gHistoryDataSource, "seq", "urn:history:root", false);
    historyContainer.RemoveElement(RDF.GetResource(resource), true);
    // Construct object to remove item from multizilla.rdf
    var obj = new Object();
    obj.index = Number(index);
    obj.resource = selectedResource;
    obj.property = mzRDF + "history-" + obj.index;
    // Add item to pending removal queue (removal will be done in function storeDataCollection)
    gHistoryPropertiesToBeRemoved.push(obj);
  }
  // Are all items removed?
  if (historyContainer.GetCount() < 1) {
    // Yes, remove history container and disable tree
    removeDataResources(gHistoryDataSource, "urn:history:root");
    gHistoryTree.setAttribute("disabled", "true");
  }
  mzFlushDataSource(gHistoryDataSource);
  refreshFocus();
  hasModifiedSettings();
}


